@@ -32,7 +32,6 @@ from magnum.api.controllers.v1 import types
 from magnum.api import expose
 from magnum.api import utils as api_utils
 from magnum.api import validation
-from magnum.common import clients
 from magnum.common import exception
 from magnum.common import name_generator
 from magnum.common import policy
@@ -397,22 +396,11 @@ class ClustersController(base.Controller):
 
         and store them into cluster.faults.
         """
-        osc = clients.OpenStackClients(context)
-        filters = {'status': 'FAILED'}
-        try:
-            failed_resources = osc.heat().resources.list(
-                cluster.stack_id, nested_depth=2, filters=filters)
-        except Exception as e:
-            failed_resources = []
-            LOG.warning("Failed to retrieve failed resources for "
-                        "cluster %(cluster)s from Heat stack "
-                        "%(stack)s due to error: %(e)s",
-                        {'cluster': cluster.uuid,
-                         'stack': cluster.stack_id, 'e': e},
-                        exc_info=True)
-
-        return {res.resource_name: res.resource_status_reason
-                for res in failed_resources}
+        # Gather fault info from the cluster nodegroups.
+        return {
+            ng.name: ng.status_reason for ng in cluster.nodegroups
+            if ng.status.endswith('FAILED')
+        }
 
     @expose.expose(Cluster, types.uuid_or_name)
     def get_one(self, cluster_ident):
@@ -437,12 +425,12 @@ class ClustersController(base.Controller):
         policy.enforce(context, 'cluster:get', cluster.as_dict(),
                        action='cluster:get')
 
-        cluster = Cluster.convert_with_links(cluster)
+        api_cluster = Cluster.convert_with_links(cluster)
 
-        if cluster.status in fields.ClusterStatus.STATUS_FAILED:
-            cluster.faults = self._collect_fault_info(context, cluster)
+        if api_cluster.status in fields.ClusterStatus.STATUS_FAILED:
+            api_cluster.faults = self._collect_fault_info(context, cluster)
 
-        return cluster
+        return api_cluster
 
     def _check_cluster_quota_limit(self, context):
         try:
@@ -536,8 +524,12 @@ class ClustersController(base.Controller):
         :param cluster_ident: UUID or logical name of a cluster.
         :param patch: a json PATCH document to apply to this cluster.
         """
-        cluster, node_count = self._patch(cluster_ident, patch)
-        pecan.request.rpcapi.cluster_update_async(cluster, node_count)
+        (cluster, node_count,
+         health_status,
+         health_status_reason) = self._patch(cluster_ident, patch)
+        pecan.request.rpcapi.cluster_update_async(cluster, node_count,
+                                                  health_status,
+                                                  health_status_reason)
         return ClusterID(cluster.uuid)
 
     @base.Controller.api_version("1.3")  # noqa
@@ -551,8 +543,12 @@ class ClustersController(base.Controller):
         :param rollback: whether to rollback cluster on update failure.
         :param patch: a json PATCH document to apply to this cluster.
         """
-        cluster, node_count = self._patch(cluster_ident, patch)
+        (cluster, node_count,
+         health_status,
+         health_status_reason) = self._patch(cluster_ident, patch)
         pecan.request.rpcapi.cluster_update_async(cluster, node_count,
+                                                  health_status,
+                                                  health_status_reason,
                                                   rollback)
         return ClusterID(cluster.uuid)
 
@@ -566,6 +562,8 @@ class ClustersController(base.Controller):
         cluster = api_utils.get_resource('Cluster', cluster_ident)
         policy.enforce(context, 'cluster:update', cluster.as_dict(),
                        action='cluster:update')
+        policy.enforce(context, "cluster:update_health_status",
+                       action="cluster:update_health_status")
         try:
             cluster_dict = cluster.as_dict()
             new_cluster = Cluster(**api_utils.apply_jsonpatch(cluster_dict,
@@ -583,7 +581,8 @@ class ClustersController(base.Controller):
                 delta.add(field)
 
         validation.validate_cluster_properties(delta)
-        return cluster, new_cluster.node_count
+        return (cluster, new_cluster.node_count,
+                new_cluster.health_status, new_cluster.health_status_reason)
 
     @expose.expose(None, types.uuid_or_name, status_code=204)
     def delete(self, cluster_ident):
