@@ -12,6 +12,9 @@ data:
   traefik.toml: |-
     logLevel = "INFO"
     defaultEntryPoints = ["http", "https"]
+    [metrics]
+      [metrics.prometheus]
+        entryPoint = "metrics"
     [api]
     [kubernetes]
     [entryPoints]
@@ -19,6 +22,8 @@ data:
         address = ":80"
       [entryPoints.https]
         address = ":443"
+      [entryPoints.metrics]
+        address = ":8082"
         [entryPoints.https.tls]
           cipherSuites = [
             "TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305",
@@ -41,13 +46,17 @@ data:
           ]
 ---
 kind: DaemonSet
-apiVersion: extensions/v1beta1
+apiVersion: apps/v1
 metadata:
   name: ingress-traefik
   namespace: kube-system
   labels:
     k8s-app: ingress-traefik-backend
 spec:
+  selector:
+    matchLabels:
+      k8s-app: ingress-traefik-backend
+      name: ingress-traefik-backend
   template:
     metadata:
       labels:
@@ -69,6 +78,8 @@ spec:
           hostPort: 443
         - name: admin
           containerPort: 8080
+        - name: metrics
+          containerPort: 8082
         securityContext:
           capabilities:
             drop:
@@ -91,6 +102,8 @@ apiVersion: v1
 metadata:
   name: ingress-traefik
   namespace: kube-system
+  labels:
+    k8s-app: traefik
 spec:
   selector:
     k8s-app: ingress-traefik-backend
@@ -104,6 +117,10 @@ spec:
     - name: admin
       protocol: TCP
       port: 8080
+    - name: metrics
+      port: 9100
+      protocol: TCP
+      targetPort: metrics
 ---
 kind: ClusterRole
 apiVersion: rbac.authorization.k8s.io/v1beta1
@@ -149,6 +166,7 @@ metadata:
   namespace: kube-system
 EOF
 )
+
 writeFile $INGRESS_TRAEFIK_MANIFEST "$INGRESS_TRAEFIK_MANIFEST_CONTENT"
 
 INGRESS_TRAEFIK_BIN="/srv/magnum/kubernetes/bin/ingress-traefik"
@@ -177,10 +195,13 @@ INGRESS_TRAEFIK_SERVICE_CONTENT='''[Unit]
 Requires=kube-apiserver.service
 
 [Service]
-Type=oneshot
-Environment=HOME=/root
+User=root
+Group=root
+Type=simple
+Restart=on-failure
 EnvironmentFile=-/etc/kubernetes/config
-ExecStart='''${INGRESS_TRAEFIK_BIN}'''
+ExecStart=/bin/bash '''${INGRESS_TRAEFIK_BIN}'''
+RestartSec=10
 
 [Install]
 WantedBy=multi-user.target
@@ -195,6 +216,7 @@ chmod 0644 ${INGRESS_TRAEFIK_SERVICE}
 
 # Launch the ingress traefik service
 set -x
-systemctl daemon-reload
-systemctl enable ingress-traefik.service
-systemctl start --no-block ingress-traefik.service
+ssh_cmd="ssh -F /srv/magnum/.ssh/config root@localhost"
+$ssh_cmd systemctl daemon-reload
+$ssh_cmd systemctl enable ingress-traefik.service
+$ssh_cmd systemctl start --no-block ingress-traefik.service
