@@ -30,61 +30,14 @@ CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
-class ServerAddressOutputMapping(template_def.NodeGroupOutputMapping):
-
-    public_ip_output_key = None
-    private_ip_output_key = None
-
-    def __init__(self, dummy_arg, nodegroup_attr=None, nodegroup_uuid=None):
-        self.nodegroup_attr = nodegroup_attr
-        self.nodegroup_uuid = nodegroup_uuid
-        self.heat_output = self.public_ip_output_key
-        self.is_stack_param = False
-
-    def set_output(self, stack, cluster_template, cluster):
-        if not cluster_template.floating_ip_enabled:
-            self.heat_output = self.private_ip_output_key
-
-        LOG.debug("Using heat_output: %s", self.heat_output)
-        super(ServerAddressOutputMapping,
-              self).set_output(stack, cluster_template, cluster)
-
-
-class MasterAddressOutputMapping(ServerAddressOutputMapping):
-    public_ip_output_key = 'kube_masters'
-    private_ip_output_key = 'kube_masters_private'
-
-
-class NodeAddressOutputMapping(ServerAddressOutputMapping):
-    public_ip_output_key = 'kube_minions'
-    private_ip_output_key = 'kube_minions_private'
-
-
 class K8sFedoraTemplateDefinition(k8s_template_def.K8sTemplateDefinition):
     """Kubernetes template for a Fedora."""
 
     def __init__(self):
         super(K8sFedoraTemplateDefinition, self).__init__()
-        self.add_parameter('docker_volume_size',
-                           cluster_attr='docker_volume_size')
         self.add_parameter('docker_storage_driver',
                            cluster_template_attr='docker_storage_driver')
 
-    def update_outputs(self, stack, cluster_template, cluster):
-        worker_ng = cluster.default_ng_worker
-        master_ng = cluster.default_ng_master
-
-        self.add_output('kube_minions',
-                        nodegroup_attr='node_addresses',
-                        nodegroup_uuid=worker_ng.uuid,
-                        mapping_type=NodeAddressOutputMapping)
-        self.add_output('kube_masters',
-                        nodegroup_attr='node_addresses',
-                        nodegroup_uuid=master_ng.uuid,
-                        mapping_type=MasterAddressOutputMapping)
-        super(K8sFedoraTemplateDefinition,
-              self).update_outputs(stack, cluster_template, cluster)
-
     def get_params(self, context, cluster_template, cluster, **kwargs):
         extra_params = kwargs.pop('extra_params', {})
 
@@ -102,7 +55,7 @@ class K8sFedoraTemplateDefinition(k8s_template_def.K8sTemplateDefinition):
                 cluster.labels.get('flannel_network_cidr', '10.100.0.0/16')
         if cluster_template.network_driver == 'calico':
             extra_params["pods_network_cidr"] = \
-                cluster.labels.get('calico_ipv4pool', '192.168.0.0/16')
+                cluster.labels.get('calico_ipv4pool', '10.100.0.0/16')
 
         # check cloud provider and cinder options. If cinder is selected,
         # the cloud provider needs to be enabled.
@@ -121,35 +74,55 @@ class K8sFedoraTemplateDefinition(k8s_template_def.K8sTemplateDefinition):
                 'to be true or unset.'))
         extra_params['cloud_provider_enabled'] = cloud_provider_enabled
 
-        extra_params['master_image'] = cluster_template.image_id
-        extra_params['minion_image'] = cluster_template.image_id
-
         label_list = ['coredns_tag',
                       'kube_tag', 'container_infra_prefix',
                       'availability_zone',
                       'cgroup_driver',
+                      'container_runtime',
+                      'containerd_version',
+                      'containerd_tarball_url',
+                      'containerd_tarball_sha256',
                       'calico_tag',
                       'calico_kube_controllers_tag', 'calico_ipv4pool',
+                      'calico_ipv4pool_ipip',
+                      'cinder_csi_enabled', 'cinder_csi_plugin_tag',
+                      'csi_attacher_tag', 'csi_provisioner_tag',
+                      'csi_snapshotter_tag', 'csi_resizer_tag',
+                      'csi_node_driver_registrar_tag',
                       'etcd_tag', 'flannel_tag', 'flannel_cni_tag',
                       'cloud_provider_tag',
                       'prometheus_tag', 'grafana_tag',
                       'heat_container_agent_tag',
                       'keystone_auth_enabled', 'k8s_keystone_auth_tag',
+                      'heapster_enabled',
+                      'metrics_server_enabled',
+                      'metrics_server_chart_tag',
                       'monitoring_enabled',
                       'prometheus_operator_chart_tag',
+                      'prometheus_adapter_enabled',
+                      'prometheus_adapter_chart_tag',
+                      'prometheus_adapter_configmap',
+                      'selinux_mode',
                       'tiller_enabled',
                       'tiller_tag',
                       'tiller_namespace',
+                      'helm_client_tag',
                       'traefik_ingress_controller_tag',
                       'node_problem_detector_tag',
                       'nginx_ingress_controller_tag',
+                      'nginx_ingress_controller_chart_tag',
                       'auto_healing_enabled', 'auto_scaling_enabled',
                       'auto_healing_controller', 'magnum_auto_healer_tag',
                       'draino_tag', 'autoscaler_tag',
-                      'min_node_count', 'max_node_count', 'npd_enabled']
+                      'min_node_count', 'max_node_count', 'npd_enabled',
+                      'ostree_remote', 'ostree_commit',
+                      'use_podman', 'kube_image_digest',
+                      'metrics_scraper_tag']
+
+        labels = self._get_relevant_labels(cluster, kwargs)
 
         for label in label_list:
-            label_value = cluster.labels.get(label)
+            label_value = labels.get(label)
             if label_value:
                 extra_params[label] = label_value
 
@@ -161,11 +134,13 @@ class K8sFedoraTemplateDefinition(k8s_template_def.K8sTemplateDefinition):
             csr_keys["private_key"].replace("\n", "\\n")
 
         extra_params['project_id'] = cluster.project_id
+        extra_params['post_install_manifest_url'] = \
+            CONF.kubernetes.post_install_manifest_url
 
         if not extra_params.get('max_node_count'):
             extra_params['max_node_count'] = cluster.node_count + 1
 
-        self._set_cert_manager_params(cluster, extra_params)
+        self._set_cert_manager_params(context, cluster, extra_params)
         self._get_keystone_auth_default_policy(extra_params)
         self._set_volumes(context, cluster, extra_params)
 
@@ -174,11 +149,12 @@ class K8sFedoraTemplateDefinition(k8s_template_def.K8sTemplateDefinition):
                                       extra_params=extra_params,
                                       **kwargs)
 
-    def _set_cert_manager_params(self, cluster, extra_params):
+    def _set_cert_manager_params(self, context, cluster, extra_params):
         cert_manager_api = cluster.labels.get('cert_manager_api')
         if strutils.bool_from_string(cert_manager_api):
             extra_params['cert_manager_api'] = cert_manager_api
-            ca_cert = cert_manager.get_cluster_ca_certificate(cluster)
+            ca_cert = cert_manager.get_cluster_ca_certificate(cluster,
+                                                              context=context)
             if six.PY3 and isinstance(ca_cert.get_private_key_passphrase(),
                                       six.text_type):
                 extra_params['ca_key'] = x509.decrypt_key(
@@ -221,33 +197,41 @@ class K8sFedoraTemplateDefinition(k8s_template_def.K8sTemplateDefinition):
 
     def _set_volumes(self, context, cluster, extra_params):
         # set docker_volume_type
-        docker_volume_type = cluster.labels.get(
+        docker_volume_size = cluster.docker_volume_size or 0
+        docker_volume_type = (cluster.labels.get(
             'docker_volume_type',
             cinder.get_default_docker_volume_type(context))
+            if int(docker_volume_size) > 0 else '')
         extra_params['docker_volume_type'] = docker_volume_type
 
         # set etcd_volume_type
-        etcd_volume_type = cluster.labels.get(
-            'etcd_volume_type', cinder.get_default_etcd_volume_type(context))
+        etcd_volume_size = cluster.labels.get('etcd_volume_size', 0)
+        etcd_volume_type = (cluster.labels.get(
+            'etcd_volume_type',
+            cinder.get_default_etcd_volume_type(context))
+            if int(etcd_volume_size) > 0 else '')
         extra_params['etcd_volume_type'] = etcd_volume_type
 
-        # set boot_volume_type
-        boot_volume_type = cluster.labels.get(
-            'boot_volume_type', cinder.get_default_boot_volume_type(context))
-        extra_params['boot_volume_type'] = boot_volume_type
-
         # set boot_volume_size
         boot_volume_size = cluster.labels.get(
             'boot_volume_size', CONF.cinder.default_boot_volume_size)
         extra_params['boot_volume_size'] = boot_volume_size
 
-    def get_env_files(self, cluster_template, cluster):
+        # set boot_volume_type
+        boot_volume_type = (cluster.labels.get(
+            'boot_volume_type',
+            cinder.get_default_boot_volume_type(context))
+            if int(boot_volume_size) > 0 else '')
+        extra_params['boot_volume_type'] = boot_volume_type
+
+    def get_env_files(self, cluster_template, cluster, nodegroup=None):
         env_files = []
 
         template_def.add_priv_net_env_file(env_files, cluster_template,
                                            cluster)
         template_def.add_etcd_volume_env_file(env_files, cluster)
-        template_def.add_volume_env_file(env_files, cluster)
+        template_def.add_volume_env_file(env_files, cluster,
+                                         nodegroup=nodegroup)
         template_def.add_lb_env_file(env_files, cluster_template)
         template_def.add_fip_env_file(env_files, cluster_template, cluster)
 
