@@ -23,6 +23,7 @@ import six
 from magnum.common import clients
 from magnum.common import exception
 from magnum.common import keystone
+from magnum.common import nova
 from magnum.common import utils
 import magnum.conf
 
@@ -92,7 +93,7 @@ class NodeGroupParameterMapping(ParameterMapping):
     def get_value(self, cluster_template, cluster):
         value = None
         for ng in cluster.nodegroups:
-            if ng.uuid == self.nodegroup_uuid:
+            if ng.uuid == self.nodegroup_uuid and self.nodegroup_attr in ng:
                 value = getattr(ng, self.nodegroup_attr)
                 break
         return value
@@ -158,6 +159,10 @@ class NodeGroupOutputMapping(OutputMapping):
                 # nodegroups are fetched from the database every
                 # time, so the bad thing here is that we need to
                 # save each change.
+                previous_value = getattr(ng, self.nodegroup_attr, None)
+                if previous_value == output_value:
+                    # Avoid saving if it's not needed.
+                    return
                 setattr(ng, self.nodegroup_attr, output_value)
                 ng.save()
 
@@ -187,6 +192,7 @@ class TemplateDefinition(object):
     def __init__(self):
         self.param_mappings = list()
         self.output_mappings = list()
+        self.nodegroup_output_mappings = list()
 
     def add_parameter(self, *args, **kwargs):
         param_class = kwargs.pop('param_class', ParameterMapping)
@@ -196,7 +202,10 @@ class TemplateDefinition(object):
     def add_output(self, *args, **kwargs):
         mapping_type = kwargs.pop('mapping_type', OutputMapping)
         output = mapping_type(*args, **kwargs)
-        self.output_mappings.append(output)
+        if kwargs.get('cluster_attr', None):
+            self.output_mappings.append(output)
+        else:
+            self.nodegroup_output_mappings.append(output)
 
     def get_output(self, *args, **kwargs):
         for output in self.output_mappings:
@@ -226,7 +235,7 @@ class TemplateDefinition(object):
 
         return template_params
 
-    def get_env_files(self, cluster_template, cluster):
+    def get_env_files(self, cluster_template, cluster, nodegroup=None):
         """Gets stack environment files based upon ClusterTemplate attributes.
 
         Base implementation returns no files (empty list). Meant to be
@@ -295,12 +304,15 @@ class TemplateDefinition(object):
     def resolve_ambiguous_values(self, context, heat_param, heat_value, value):
         return str(value)
 
-    def add_nodegroup_params(self, cluster):
+    def add_nodegroup_params(self, cluster, nodegroups=None):
         pass
 
-    def update_outputs(self, stack, cluster_template, cluster):
+    def update_outputs(self, stack, cluster_template, cluster,
+                       nodegroups=None):
         for output in self.output_mappings:
             output.set_output(stack, cluster_template, cluster)
+        for output in self.nodegroup_output_mappings:
+            output.set_output(stack, cluster_template, cluster)
 
     @abc.abstractproperty
     def driver_module_path(self):
@@ -311,9 +323,12 @@ class TemplateDefinition(object):
         pass
 
     def extract_definition(self, context, cluster_template, cluster, **kwargs):
+        nodegroups_list = kwargs.get('nodegroups', None)
+        nodegroup = None if not nodegroups_list else nodegroups_list[0]
         return (self.template_path,
                 self.get_params(context, cluster_template, cluster, **kwargs),
-                self.get_env_files(cluster_template, cluster))
+                self.get_env_files(cluster_template, cluster,
+                                   nodegroup=nodegroup))
 
 
 class BaseTemplateDefinition(TemplateDefinition):
@@ -323,8 +338,6 @@ class BaseTemplateDefinition(TemplateDefinition):
 
         self.add_parameter('ssh_key_name',
                            cluster_attr='keypair')
-        self.add_parameter('server_image',
-                           cluster_template_attr='image_id')
         self.add_parameter('dns_nameserver',
                            cluster_template_attr='dns_nameserver')
         self.add_parameter('http_proxy',
@@ -350,8 +363,9 @@ class BaseTemplateDefinition(TemplateDefinition):
     def get_params(self, context, cluster_template, cluster, **kwargs):
         osc = self.get_osc(context)
 
+        nodegroups = kwargs.pop('nodegroups', None)
         # Add all the params from the cluster's nodegroups
-        self.add_nodegroup_params(cluster)
+        self.add_nodegroup_params(cluster, nodegroups=nodegroups)
 
         extra_params = kwargs.pop('extra_params', {})
         extra_params['trustee_domain_id'] = osc.keystone().trustee_domain_id
@@ -360,6 +374,9 @@ class BaseTemplateDefinition(TemplateDefinition):
         extra_params['trustee_password'] = cluster.trustee_password
         extra_params['verify_ca'] = CONF.drivers.verify_ca
         extra_params['openstack_ca'] = utils.get_openstack_ca()
+        ssh_public_key = nova.get_ssh_key(context, cluster.keypair)
+        if ssh_public_key != "":
+            extra_params['ssh_public_key'] = ssh_public_key
 
         # Only pass trust ID into the template if allowed by the config file
         if CONF.trust.cluster_user_trust:
@@ -394,25 +411,51 @@ class BaseTemplateDefinition(TemplateDefinition):
         return super(BaseTemplateDefinition, self).resolve_ambiguous_values(
             context, heat_param, heat_value, value)
 
-    def add_nodegroup_params(self, cluster):
-        # Assuming that all the drivers that will not override
-        # this method do not support more than two nodegroups.
-        # Meaning that we have one master and one worker.
-        master_ng = cluster.default_ng_master
-        self.add_parameter('number_of_masters',
-                           nodegroup_attr='node_count',
-                           nodegroup_uuid=master_ng.uuid,
-                           param_class=NodeGroupParameterMapping)
+    def add_nodegroup_params(self, cluster, nodegroups=None):
+        master_params, worker_params = self.get_nodegroup_param_maps()
+        nodegroups = nodegroups or [cluster.default_ng_worker,
+                                    cluster.default_ng_master]
+        for nodegroup in nodegroups:
+            params = worker_params
+            if nodegroup.role == 'master':
+                params = master_params
+            self._handle_nodegroup_param_map(nodegroup, params)
 
-    def update_outputs(self, stack, cluster_template, cluster):
+    def get_nodegroup_param_maps(self, master_params=None, worker_params=None):
+        master_params = master_params or dict()
+        worker_params = worker_params or dict()
+        master_params.update({
+            'number_of_masters': 'node_count',
+        })
+        return master_params, worker_params
+
+    def _handle_nodegroup_param_map(self, nodegroup, param_map):
+        for template_attr, nodegroup_attr in param_map.items():
+            self.add_parameter(template_attr, nodegroup_attr=nodegroup_attr,
+                               nodegroup_uuid=nodegroup.uuid,
+                               param_class=NodeGroupParameterMapping)
+
+    def _get_relevant_labels(self, cluster, kwargs):
+        nodegroups = kwargs.get('nodegroups', None)
+        labels = cluster.labels
+        if nodegroups is not None:
+            labels = nodegroups[0].labels
+        return labels
+
+    def update_outputs(self, stack, cluster_template, cluster,
+                       nodegroups=None):
         master_ng = cluster.default_ng_master
-        self.add_output('number_of_masters',
-                        nodegroup_attr='node_count',
-                        nodegroup_uuid=master_ng.uuid,
-                        is_stack_param=True,
-                        mapping_type=NodeGroupOutputMapping)
+        nodegroups = nodegroups or [cluster.default_ng_master]
+        for nodegroup in nodegroups:
+            if nodegroup.role == 'master':
+                self.add_output('number_of_masters',
+                                nodegroup_attr='node_count',
+                                nodegroup_uuid=master_ng.uuid,
+                                is_stack_param=True,
+                                mapping_type=NodeGroupOutputMapping)
         super(BaseTemplateDefinition,
-              self).update_outputs(stack, cluster_template, cluster)
+              self).update_outputs(stack, cluster_template, cluster,
+                                   nodegroups=nodegroups)
 
     def validate_discovery_url(self, discovery_url, expect_size):
         url = str(discovery_url)
@@ -493,8 +536,12 @@ def add_lb_env_file(env_files, cluster_template):
         env_files.append(COMMON_ENV_PATH + 'no_master_lb.yaml')
 
 
-def add_volume_env_file(env_files, cluster):
-    if cluster.docker_volume_size is None:
+def add_volume_env_file(env_files, cluster, nodegroup=None):
+    if nodegroup:
+        docker_volume_size = nodegroup.docker_volume_size
+    else:
+        docker_volume_size = cluster.docker_volume_size
+    if docker_volume_size is None:
         env_files.append(COMMON_ENV_PATH + 'no_volume.yaml')
     else:
         env_files.append(COMMON_ENV_PATH + 'with_volume.yaml')
@@ -508,11 +555,9 @@ def add_etcd_volume_env_file(env_files, cluster):
 
 
 def add_fip_env_file(env_files, cluster_template, cluster):
-    lb_fip_enabled = cluster.labels.get(
-        "master_lb_floating_ip_enabled",
-        cluster.floating_ip_enabled
-    )
-    master_lb_fip_enabled = strutils.bool_from_string(lb_fip_enabled)
+    lb_fip_enabled = cluster.labels.get("master_lb_floating_ip_enabled")
+    master_lb_fip_enabled = (strutils.bool_from_string(lb_fip_enabled) or
+                             cluster.floating_ip_enabled)
 
     if cluster.floating_ip_enabled:
         env_files.append(COMMON_ENV_PATH + 'enable_floating_ip.yaml')
