@@ -89,30 +89,15 @@ class TestListCluster(api_base.FunctionalTest):
         self.assertEqual(cluster.uuid, response['uuid'])
         self._verify_attrs(self._expand_cluster_attrs, response)
 
-    @mock.patch('magnum.common.clients.OpenStackClients.heat')
-    def test_get_one_failed_cluster(self, mock_heat):
-        fake_resources = mock.MagicMock()
-        fake_resources.resource_name = 'fake_name'
-        fake_resources.resource_status_reason = 'fake_reason'
-
-        ht = mock.MagicMock()
-        ht.resources.list.return_value = [fake_resources]
-        mock_heat.return_value = ht
-
+    def test_get_one_failed_cluster(self):
         cluster = obj_utils.create_test_cluster(self.context,
-                                                status='CREATE_FAILED')
+                                                status='CREATE_FAILED',
+                                                master_status='CREATE_FAILED',
+                                                master_reason='fake_reason')
         response = self.get_json('/clusters/%s' % cluster['uuid'])
+        expected_faults = {cluster.default_ng_master.name: 'fake_reason'}
         self.assertEqual(cluster.uuid, response['uuid'])
-        self.assertEqual({'fake_name': 'fake_reason'}, response['faults'])
-
-    @mock.patch('magnum.common.clients.OpenStackClients.heat')
-    def test_get_one_failed_cluster_heatclient_exception(self, mock_heat):
-        mock_heat.resources.list.side_effect = Exception('fake')
-        cluster = obj_utils.create_test_cluster(self.context,
-                                                status='CREATE_FAILED')
-        response = self.get_json('/clusters/%s' % cluster['uuid'])
-        self.assertEqual(cluster.uuid, response['uuid'])
-        self.assertEqual({}, response['faults'])
+        self.assertEqual(expected_faults, response['faults'])
 
     def test_get_one_by_name(self):
         cluster = obj_utils.create_test_cluster(self.context)
@@ -274,13 +259,15 @@ class TestPatch(api_base.FunctionalTest):
         self.cluster_template_obj = obj_utils.create_test_cluster_template(
             self.context)
         self.cluster_obj = obj_utils.create_test_cluster(
-            self.context, name='cluster_example_A', node_count=3)
+            self.context, name='cluster_example_A', node_count=3,
+            health_status='UNKNOWN', health_status_reason={})
         p = mock.patch.object(rpcapi.API, 'cluster_update_async')
         self.mock_cluster_update = p.start()
         self.mock_cluster_update.side_effect = self._sim_rpc_cluster_update
         self.addCleanup(p.stop)
 
-    def _sim_rpc_cluster_update(self, cluster, node_count, rollback=False):
+    def _sim_rpc_cluster_update(self, cluster, node_count, health_status,
+                                health_status_reason, rollback=False):
         cluster.status = 'UPDATE_IN_PROGRESS'
         default_ng_worker = cluster.default_ng_worker
         default_ng_worker.node_count = node_count
@@ -449,7 +436,8 @@ class TestPatch(api_base.FunctionalTest):
             headers={'OpenStack-API-Version': 'container-infra 1.3'})
 
         self.mock_cluster_update.assert_called_once_with(
-            mock.ANY, node_count, True)
+            mock.ANY, node_count, self.cluster_obj.health_status,
+            self.cluster_obj.health_status_reason, True)
         self.assertEqual(202, response.status_code)
 
     def test_update_cluster_with_rollback_disabled(self):
@@ -461,7 +449,8 @@ class TestPatch(api_base.FunctionalTest):
             headers={'OpenStack-API-Version': 'container-infra 1.3'})
 
         self.mock_cluster_update.assert_called_once_with(
-            mock.ANY, node_count, False)
+            mock.ANY, node_count, self.cluster_obj.health_status,
+            self.cluster_obj.health_status_reason, False)
         self.assertEqual(202, response.status_code)
 
     def test_remove_ok(self):
