@@ -47,11 +47,15 @@ class MonitorsTestCase(base.TestCase):
 
     def setUp(self):
         super(MonitorsTestCase, self).setUp()
-
         cluster = utils.get_test_cluster(node_addresses=['1.2.3.4'],
                                          api_address='https://5.6.7.8:2376',
-                                         master_addresses=['10.0.0.6'])
+                                         master_addresses=['10.0.0.6'],
+                                         labels={})
         self.cluster = objects.Cluster(self.context, **cluster)
+        cluster_template = (
+            utils.get_test_cluster_template(master_lb_enabled=False))
+        self.cluster.cluster_template = (
+            objects.ClusterTemplate(self.context, **cluster_template))
         nodegroups = utils.get_nodegroups_for_cluster(
             node_addresses=['1.2.3.4'], master_addresses=['10.0.0.6'])
         self.nodegroups = [
@@ -511,3 +515,51 @@ class MonitorsTestCase(base.TestCase):
         self.assertEqual(self.k8s_monitor.data['health_status_reason'],
                          {'api': 'ok', 'k8s-cluster-node-0.Ready': False,
                           'api': 'ok', 'k8s-cluster-node-1.Ready': True})
+
+    @mock.patch('magnum.conductor.k8s_api.create_k8s_api')
+    def test_k8s_monitor_health_unreachable_cluster(self, mock_k8s_api):
+        mock_nodes = mock.MagicMock()
+        mock_node = mock.MagicMock()
+        mock_node.status = mock.MagicMock()
+        mock_nodes.items = [mock_node]
+        self.k8s_monitor.cluster.floating_ip_enabled = False
+
+        self.k8s_monitor.poll_health_status()
+        self.assertEqual(self.k8s_monitor.data['health_status'],
+                         m_fields.ClusterHealthStatus.UNKNOWN)
+
+    @mock.patch('magnum.conductor.k8s_api.create_k8s_api')
+    def test_k8s_monitor_health_unreachable_with_master_lb(self, mock_k8s_api):
+        mock_nodes = mock.MagicMock()
+        mock_node = mock.MagicMock()
+        mock_node.status = mock.MagicMock()
+        mock_nodes.items = [mock_node]
+        cluster = self.k8s_monitor.cluster
+        cluster.floating_ip_enabled = True
+        cluster.cluster_template.master_lb_enabled = True
+        cluster.labels['master_lb_floating_ip_enabled'] = False
+
+        self.k8s_monitor.poll_health_status()
+        self.assertEqual(self.k8s_monitor.data['health_status'],
+                         m_fields.ClusterHealthStatus.UNKNOWN)
+
+    def test_is_magnum_auto_healer_running(self):
+        cluster = self.k8s_monitor.cluster
+        cluster.labels['auto_healing_enabled'] = True
+        cluster.labels['auto_healing_controller'] = 'magnum-auto-healer'
+        self.k8s_monitor._is_magnum_auto_healer_running()
+        self.assertTrue(self.k8s_monitor._is_magnum_auto_healer_running())
+
+        cluster.labels['auto_healing_enabled'] = False
+        cluster.labels['auto_healing_controller'] = 'magnum-auto-healer'
+        self.k8s_monitor._is_magnum_auto_healer_running()
+        self.assertFalse(self.k8s_monitor._is_magnum_auto_healer_running())
+
+        cluster.labels['auto_healing_enabled'] = True
+        cluster.labels['auto_healing_controller'] = 'draino'
+        self.k8s_monitor._is_magnum_auto_healer_running()
+        self.assertFalse(self.k8s_monitor._is_magnum_auto_healer_running())
+
+        cluster.labels = {}
+        self.k8s_monitor._is_magnum_auto_healer_running()
+        self.assertFalse(self.k8s_monitor._is_magnum_auto_healer_running())
